<?php
require_once(__DIR__.'/settings.php');
require_once(VMNC_IEM_ROOT.'/admin/functions/api/jobs_send.php');

use vimmaniac\msh as msh;
use vimmaniac\mpl as mpl;

class Jobs_Send_API_MSH extends Jobs_Send_API {
    function
    ActionJob($jobid=0, $queueid=0){
        global
            $VMNC_MSH_logger,
            $VMNC_MSH_enabled,
            $VMNC_MSH_ipc_host,
            $VMNC_MSH_ipc_port,

            $tso,
            $VMNC_MSH_php_cli_binary_path,
            $VMNC_MSH_child_script_path,

            $VMNC_MSH_n_children,
            $VMNC_MSH_task_chunk_size,
            $VMNC_MSH_parent_poll_period;

        if (! $VMNC_MSH_enabled)
            { return parent::ActionJob($jobid, $queueid); }

        if (!$this->SetupJob($jobid, $queueid)) {
            return false;
        }

        $this->NotifyOwner();

        $tso = new msh\TaskSupplierMSH($this, $jobid, $queueid);
        $ppm = new mpl\ParentProcessManagerSoq(
            $VMNC_MSH_ipc_host,
            $VMNC_MSH_ipc_port,

            $tso,
            $VMNC_MSH_php_cli_binary_path,
            $VMNC_MSH_child_script_path,

            $VMNC_MSH_n_children,
            $VMNC_MSH_task_chunk_size,
            $VMNC_MSH_parent_poll_period
        );

        $VMNC_MSH_logger->add(
            "Executing jobid: {$jobid}, queueid: {$queueid} ..."
            , VMNC_LOG_LEVEL_INFO
        );

        $paused = $ppm->start();

        $VMNC_MSH_logger->add(
            ($paused) ? "Job was paused." : "Job completed successfully."
            , VMNC_LOG_LEVEL_INFO
        );

        $VMNC_MSH_logger->add(
            sprintf(
                "Send successful: %s, Failures: %s"
                , $tso->n_success, $tso->n_failure
            )
            , VMNC_LOG_LEVEL_INFO
        );
        

         if ($paused) {
            $finished = false;
            $this->jobstatus = 'p';
        } else {
            $this->jobstatus = 'c';
            $finished = true;
            $this->CleanUp($queueid);
        }

        $this->Email_API->SMTP_Logout();
        $this->NotifyOwner();
        $this->ResetSend();

        return $finished;
    }

    function
    msh_update_jobdetails($queueid, $n_success, $n_failure){
        global $VMNC_MSH_logger, $VMNC_LOG_extra;

        if ($VMNC_LOG_extra > 0){
            $VMNC_MSH_logger->add(
                "Updating 'jobdetails' in database ...",
                VMNC_LOG_LEVEL_DEBUG
            );
        }

        $this->jobdetails['EmailResults']['success'] = $n_success;
        $this->jobdetails['EmailResults']['failure'] = $n_failure;
        $this->jobdetails['EmailResults']['total'] = $n_failure + $n_success;

        $query =
            "UPDATE [|PREFIX|]jobs SET jobdetails='"
            . $this->Db->Quote(serialize($this->jobdetails))
            . "' WHERE queueid={$queueid}";

        $this->Db->Query($query);
    }

    function
    msh_update_lastupdatetime($jobid){
        global $VMNC_MSH_logger, $VMNC_LOG_extra;

        if ($VMNC_LOG_extra > 0){
            $VMNC_MSH_logger->add(
                "Updating 'lastupdatetime' in database ...",
                VMNC_LOG_LEVEL_DEBUG
            );
        }

        $query = "UPDATE " . SENDSTUDIO_TABLEPREFIX
                 . "jobs SET lastupdatetime='"
                 . $this->GetServerTime()
                 . "' WHERE jobid='" . (int)$jobid . "'";

        $this->Db->Query($query);
    }

    ##########################################################################
    #          IEM SOURCE CODE WITH SOME SECTIONS COMMENTED OUT 
    ##########################################################################
	function SendToRecipient($recipient = 0, $queueid = 0, $queuetype = 'send')
	{
		$this->Email_API->ClearRecipients();
		
		$this->jobdetails['EmailResults']['total']++;
				
		$subscriberinfo = array();
		$subscriberinfo = $this->Subscriber_API->LoadSubscriberBasicInformation($recipient, $this->jobdetails['Lists']);
		
		// Removes subsriber from queue if we can't find it's record from list_subscribers table
		// Do not add this to queues_unsent table.
		if (empty($subscriberinfo) || !isset($subscriberinfo['subscribedate'])) {
			$this->RemoveFromQueue($queueid, $queuetype, $recipient);
			$mail_result['success'] = 0;
			$mail_result['fail'] = array();
			$mail_result['fail'][] = array($recipient, 'Unable to load subscriber info or they do not have a subscribe date set.');
			#$this->jobdetails['EmailResults']['failure']++;
			#$query = "UPDATE [|PREFIX|]jobs SET jobdetails='" . $this->Db->Quote(serialize($this->jobdetails)) . "' WHERE queueid={$queueid}";
			#$result = $this->Db->Query($query);
			#if (!$result) {trigger_error(mysql_error());}
			return $mail_result;
		}

		if (sizeof($this->jobdetails['Lists']) == 1) {
			if (empty($this->listnames)) {
				$listid = current($this->jobdetails['Lists']);
				$this->Lists_API->Load($listid);
				$this->listnames = array (
					$listid => array (
						'listname' => $this->Lists_API->Get('name'),
						'companyname' => $this->Lists_API->Get('companyname'),
						'companyaddress' => $this->Lists_API->Get('companyaddress'),
						'companyphone' => $this->Lists_API->Get('companyphone'),
					)
				);
			}
		} else {
			// a subscriberid is unique per list and per system, so a subscriberid can only have one listid associated with it.
			// so we'll just get the listname from that.
			$listid = $subscriberinfo['listid'];
			if (!in_array($listid, array_keys($this->listnames))) {
				$this->Lists_API->Load($listid);
				$this->listnames[$listid] = array (
					'listname' => $this->Lists_API->Get('name'),
					'companyname' => $this->Lists_API->Get('companyname'),
					'companyaddress' => $this->Lists_API->Get('companyaddress'),
					'companyphone' => $this->Lists_API->Get('companyphone'),
				);
			}
		}

		$subscriberinfo['ipaddress'] = $subscriberinfo['confirmip'];
		if (!$subscriberinfo['ipaddress']) {
			$subscriberinfo['ipaddress'] = $subscriberinfo['requestip'];
		}
		if (!$subscriberinfo['ipaddress']) {
			$subscriberinfo['ipaddress'] = 'n/a';
		}

		$subscriberinfo['CustomFields'] = array();
		if (isset($this->all_customfields[$recipient])) {
			$subscriberinfo['CustomFields'] = $this->all_customfields[$recipient];
		} else {
			/**
			* If the subscriber has no custom fields coming from the database, then set up blank placeholders.
			* If they have no custom fields in the database, they have no records in the 'all_customfields' array - so we need to fill it up with blank entries.
			*/
			foreach ($this->custom_fields_to_replace as $fieldid => $fieldname) {
				$subscriberinfo['CustomFields'][] = array(
					'fieldid' => $fieldid,
					'fieldname' => $fieldname,
					'fieldtype' => 'text',
					'defaultvalue' => '',
					'fieldsettings' => '',
					'subscriberid' => $recipient,
					'data' => ''
				);
			}
		}

		$name = false;

		if ($this->jobdetails['To_FirstName']) {
			foreach ($subscriberinfo['CustomFields'] as $p => $details) {
				if ($details['fieldid'] == $this->jobdetails['To_FirstName'] && $details['data'] != '') {
					$name = $details['data'];
					break;
				}
			}
		}

		if ($this->jobdetails['To_LastName']) {
			foreach ($subscriberinfo['CustomFields'] as $p => $details) {
				if ($details['fieldid'] == $this->jobdetails['To_LastName'] && $details['data'] != '') {
					$name .= ' ' . $details['data'];
					break;
				}
			}
		}

		$send_format = $subscriberinfo['format'];
		if ($this->newsletter['Format'] == 't') {
			$send_format = 't';
		}

		$this->Email_API->AddRecipient($subscriberinfo['emailaddress'], $name, $send_format, $subscriberinfo['subscriberid']);

		$lid = $subscriberinfo['listid'];
		$subscriberinfo['listname'] = $this->listnames[$lid]['listname'];
		$subscriberinfo['companyname'] = $this->listnames[$lid]['companyname'];
		$subscriberinfo['companyaddress'] = $this->listnames[$lid]['companyaddress'];
		$subscriberinfo['companyphone'] = $this->listnames[$lid]['companyphone'];

		$subscriberinfo['newsletter'] = $this->jobdetails['Newsletter'];
		$subscriberinfo['statid'] = $this->statid;

		$this->Email_API->AddCustomFieldInfo($subscriberinfo['emailaddress'], $subscriberinfo);

		$this->Email_API->AddDynamicContentInfo($this->dynamic_content_replacement);

		$disconnect = true;
		if (defined('IEM_CRON_JOB')) {
			$disconnect = false;
		}
		$mail_result = $this->Email_API->Send(true, $disconnect);

		if (!$this->Email_API->Get('Multipart')) {
			$sent_format = $subscriberinfo['format'];
		} else {
			$sent_format = 'multipart';
		}

		if ($mail_result['success'] > 0) {
			$this->Stats_API->UpdateRecipient($this->statid, $sent_format, 'n');
			$this->jobdetails['EmailResults']['success']++;
		} else {
			reset($mail_result['fail']);
			$failure_message = current($mail_result['fail']);
			switch ($failure_message[1]) {
				case 'BlankEmail':
					$errorcode = 10;
					$errormessage = $failure_message[1];
				break;

				default:
					$errorcode = 20;
					$errormessage = 'ServerRejected';

					if ($this->Email_API->SMTPServer !== false) {
						$errorcode = 30;
						$errormessage = $failure_message[1];
					}
			}

			$this->Save_Unsent_Recipient($recipient, $queueid, $errorcode, $errormessage);
			
			$this->jobdetails['EmailResults']['failure']++;
		}
		#$query = "UPDATE [|PREFIX|]jobs SET jobdetails='" . $this->Db->Quote(serialize($this->jobdetails)) . "' WHERE queueid={$queueid}";
		#$result = $this->Db->Query($query);
		#if (!$result) {trigger_error(mysql_error());}
		#$this->emailssent++;

		/*
		 * Trigger the sending event so hooks get fired.
		 */
		$tempEventData                 = new EventData_IEM_SENDAPI_SENDTORECIPIENT();
		$tempEventData->emailsent      = ($mail_result['success'] > 0);
		$tempEventData->jobdetails     = &$this->jobdetails;
		$tempEventData->subscriberinfo = &$subscriberinfo;
		$tempEventData->newsletter     = &$this->newsletter;
		$tempEventData->trigger();

		unset($tempEventData);

		if (!$this->MarkAsProcessed($queueid, $this->queuetype, $recipient)) {
			// This will make sure that we do not continue sending when the database server is DOWN
			die ('Cannot mark queue as procesed (Cannot write to the database)... Send Job stopped!! (QueueID: ' . $queueid . ', RecipientID: ' . $recipient . ')');
		}

		return $mail_result;
	}


    ##########################################################################
    #                          FIX FOR RESEND BUG
    ##########################################################################
	function FetchJob()
	{
        global $VMNC_MSH_fix_iem_resend_bug;
        if (! $VMNC_MSH_fix_iem_resend_bug)
            { return parent::FetchJob(); }

		$timenow = $this->GetServerTime();
		$half_hour_ago = $timenow - (30 * 60);

		$query = "SELECT * FROM " . SENDSTUDIO_TABLEPREFIX . "jobs WHERE jobtype='send' AND (";

		/**
		* get "waiting" jobs
		*/
		$query .= " (jobstatus ='w' AND jobtime < " . $timenow . ") OR ";

		/**
		* get "resending" jobs
		*/
		$query .= " (jobstatus='r' AND jobtime < " . $timenow . ") OR ";

		/**
		* Get jobs that haven't been updated in half an hour.
		* This is in case a job has broken (eg the db went down or server was rebooted mid-send).
		*/
		$query .= " (jobstatus='i' AND jobtime < " . $timenow . " AND lastupdatetime < " . $half_hour_ago . ")";

		/**
		* order the results so we get the one scheduled first to send.
		*/
		$query .= ") AND (approved > 0)";

		$query .= " ORDER BY jobtime ASC LIMIT 1";
		$result = $this->Db->Query($query);
		if (!$result) {
			return false;
		}

		$row = $this->Db->Fetch($result);
		if (empty($row)) {
			return false;
		}

		$query = "SELECT COUNT(*) AS count FROM " . SENDSTUDIO_TABLEPREFIX . "jobs_lists WHERE jobid='" . $row['jobid'] . "'";
		$result = $this->Db->Query($query);
		$count_check = $this->Db->FetchOne($result, 'count');
		if ($count_check <= 0) {
			echo '*** Found an orphaned job: ' . $row['jobid'] . ' *** - No entries in the jobs_lists table. Please contact your administrator.' . "\n";
			$this->PauseJob($row['jobid']);
			return false;
		}

		$this->jobstatus = $row['jobstatus'];
		$this->jobdetails = unserialize($row['jobdetails']);
		$this->jobowner = $row['ownerid'];

		// if the job is in progress, we'll pause it so the send part can pick it up properly.
		if ($row['jobstatus'] == 'i') {
			$this->PauseJob($row['jobid']);
		}

		// if the job is in 'resend' mode, we need to do some work before it will be picked up properly.
		// the job needs to be 'started' before it can be re-set up, so start it up, do the setup, then pause.
		// see GetJobQueueSize
		if ($row['jobstatus'] == 'r') {
			$start_ok = $this->StartJob($row['jobid']);
			if (!$start_ok) {
				return false;
			}

			$resend_ok = $this->ResendJob_Setup($row['jobid']);
			if (!$resend_ok) {
				return false;
			}

			$pause_ok = $this->PauseJob($row['jobid']);
			if (!$pause_ok) {
				return false;
			}
/*
			$this->Db->StartTransaction();
			$start_ok = $this->StartJob($row['jobid']);
			if (!$start_ok) {
				$this->Db->RollbackTransaction();
				return false;
			}

			$resend_ok = $this->ResendJob_Setup($row['jobid']);
			if (!$resend_ok) {
				$this->Db->RollbackTransaction();
				return false;
			}

			$pause_ok = $this->PauseJob($row['jobid']);
			if (!$pause_ok) {
				$this->Db->RollbackTransaction();
				return false;
			}

			$this->Db->CommitTransaction();
*/
		}

		return $row['jobid'];
	}
    ##########################################################################
}
