<?php
namespace vimmaniac\msh;

require_once(__DIR__.'/settings.php');
use vimmaniac\mpl as mpl;

class TaskSupplierMSH
implements mpl\TaskSupplierInterface {

    public function
    __construct($jobs_send_obj, $jobid, $queueid){
        global $VMNC_MSH_logger;

        $this->jobs_send_obj = $jobs_send_obj;
        $this->jobid = $jobid;
        $this->queueid = $queueid;

        $this->n_success = 0;
        $this->n_failure = 0;

        $VMNC_MSH_logger->add(
            sprintf(
                "'%s' started with - jobid: '''%s''', queueid: '''%s'''.",
                get_class($this), $this->jobid, $this->queueid
            )
            , VMNC_LOG_LEVEL_DEBUG
        );
    }

    public function
    do_before_start_of_split(){}

    public function
    do_after_end_of_split(){}

    public function
    do_before_start_of_join(){}

    public function
    do_after_end_of_join(){}

    public function
    supply_task_hint(){
        return array(
            'jobid' => $this->jobid,
            'queueid' => $this->queueid

        );
    }

    public function
    supply_large_task_hint(){
        $ovs = get_object_vars($this->jobs_send_obj);
        foreach ($ovs as $k => $v){
            if (is_resource($v) || is_object($v))
                { unset($ovs[$k]); }
        }

        return array(
            'jobs_send_obj_vars' => $ovs
        );
    }

    public function
    supply_new_tasks($max_n_tasks){
        $recipients = $this->jobs_send_obj->FetchFromQueue(
            $this->queueid, 'send', 1, $max_n_tasks
        );

        if (empty($recipients))
            { throw new mpl\TasksExhausted(); }

        return $recipients;
    }

    public function
    periodic_callback_A(){
        $this->jobs_send_obj->msh_update_jobdetails(
            $this->queueid, $this->n_success, $this->n_failure
        );

        $this->jobs_send_obj->msh_update_lastupdatetime($this->jobid);

        if ($this->jobs_send_obj->JobPaused($this->jobid))
            { throw new mpl\UnnaturalShutdown(); }
    }

    public function
    modify_finish_status($finish_status){
        $paused = ($finish_status == 'unnatural') ? True : False;
        return $paused;
    }

    public function
    process_task_status($task_status){
        $this->n_success += $task_status['success'];
        $this->n_failure += $task_status['failure'];
    }
}

class TaskPerformerMSH
implements mpl\TaskPerformerInterface {
    public function
    __construct($task_hint, $large_task_hint){
        $this->task_hint = $task_hint;

        $this->large_task_hint = $large_task_hint;
        $this->setup();
    }

    public function
    setup(){
        global $VMNC_MSH_logger, $VMNC_LOG_extra;

        if ($VMNC_LOG_extra > 1){
            $VMNC_MSH_logger->add(
                sprintf("Setting up '%s' ...", get_class($this))
                , VMNC_LOG_LEVEL_DEBUG
            );
        }

        $this->queueid = $this->task_hint['queueid'];
        $this->jobid = $this->task_hint['jobid'];

        $this->jobs_send_obj = new \Jobs_Send_API_MSH();
        foreach ($this->large_task_hint['jobs_send_obj_vars'] as $k=>$v){
            $this->jobs_send_obj->{$k} = $v;
        }

        $this->jobs_send_obj->SetupJob($this->jobid, $this->queueid);
        $this->_recipients_to_track = array();
    }

    public function
    teardown(){}

    public function
    _perform_dbcleanup($sent_to_recipients){
        global $VMNC_MSH_logger, $VMNC_LOG_extra;

        if ($VMNC_LOG_extra > 5){
            $VMNC_MSH_logger->add(
                sprintf(
                    "Cleaning up %s recipients from database ..."
                    , count($sent_to_recipients)
                )
                , VMNC_LOG_LEVEL_DEBUG
            );
        }

        // Fix for invalid query bug.
        if (count($sent_to_recipients) == 0) { return; }

        $query =
            "DELETE FROM " . SENDSTUDIO_TABLEPREFIX
            . "queues WHERE queueid='" . $this->queueid
            . "' AND queuetype='send' AND recipient IN ("
            . implode(',', $sent_to_recipients) . ") AND processed='1'";

        $this->jobs_send_obj->Db->StartTransaction();
        $this->jobs_send_obj->Db->Query($query);
        $this->jobs_send_obj->Db->CommitTransaction();
    }

    public function
    perform_tasks($recipients, $periodic_callback_B){
        global
            $VMNC_MSH_logger,
            $VMNC_LOG_extra,
            $VMNC_MSH_child_poll_period,
            $VMNC_MSH_enable_periodic_dbcleanup,
            $VMNC_MSH_dbcleanup_granularity;

        $this->jobs_send_obj->SetupDynamicContentFields($recipients);
        $this->jobs_send_obj->SetupCustomFields($recipients);

        $last_dbcleanup_mark = 0; 
        $recipients_to_cleanup = array();
        $last_poll_time = microtime(True);

        $task_status = array('success' => 0, 'failure' => 0);

        foreach ($recipients as $p => $recipientid){
            # Send the mail to the recipientid.
            $this->jobs_send_obj->Db->StartTransaction();
            $send_results =
                $this->jobs_send_obj->SendToRecipient(
                    $recipientid, $this->queueid
            );
            $this->jobs_send_obj->Db->CommitTransaction();

            if ($send_results['success'] > 0)
                { $task_status['success'] += 1; }
            else 
                { $task_status['failure'] += 1; }

            # Mark recipient to be cleaned up from database.
            $recipients_to_cleanup[] = $recipientid;

            # Perform periodic poll.
            $current_time = microtime(True);
            if ( ($current_time - $last_poll_time)
                  > $VMNC_MSH_child_poll_period ) {

                if ($VMNC_LOG_extra > 3){
                    $VMNC_MSH_logger->add(
                        "Performing periodic poll ..."
                        , VMNC_LOG_LEVEL_DEBUG
                    );
                }

                $last_poll_time = $current_time;

                try { call_user_func($periodic_callback_B); }
                catch (mpl\RequestedToResign $excp){
                    $this->_perform_dbcleanup($recipients_to_cleanup);
                    throw new mpl\RequestedToResign($task_status);
                }
            }

            # Perform early dbcleanup, if enabled.
            if ($VMNC_MSH_enable_periodic_dbcleanup){
                $curr_dbcleanup_mark =
                    $last_dbcleanup_mark + count($recipients_to_cleanup);

                if ( ($curr_dbcleanup_mark - $last_dbcleanup_mark)
                      > $VMNC_MSH_dbcleanup_granularity ) {

                    $last_dbcleanup_mark = $curr_dbcleanup_mark;
                    if ($VMNC_LOG_extra > 3){
                        $VMNC_MSH_logger->add(
                            "Performing early dbcleanup ..."
                            , VMNC_LOG_LEVEL_DEBUG
                        );
                    }

                    $this->_perform_dbcleanup($recipients_to_cleanup);
                    $recipients_to_cleanup = array();

                }
            }
        } 
        # Perform final dbcleanup.
        $this->_perform_dbcleanup($recipients_to_cleanup);
        return $task_status;
    }
}

